import itertools

class CleanUp:
    def __init__(self):
        self.match_percentage = ''
        self.match_count = ''
        self.saved_outcome = ''
        self.saved_match_count = ''
        self.saved_combined_length = ''
        self.combined_lenth = ''
        
    def normalize_string(self, name):
        """
        This method replaces common mistakes with letters to make it uniform and phonetically flat
        """
        name = str(name).upper()
        name = name.replace('@', 'A')
        name = name.replace('$', 'S')
        name = name.replace('0', 'O')
        name = name.replace('5', 'S')
        name = name.replace('1', 'I')
        name = name.replace('6', 'B')
        name = name.replace('Z', 'S')
        name = name.replace('OH', 'O')
        name = name.replace('CH', 'SH')
        name = name.replace('PH', 'F')
        name = name.replace('C', 'K')
        name = name.replace('J', 'G')
        name = name.replace('EI', 'IE')
        name = name.replace('AE', 'EA')
        name = name.replace('ER', 'AR')
        if name[-1] == 'E':
            name = name[:-1]
        if name[-1] == 'H':
            name = name[:-1]
        return name
        
    def keep_alphabets(self, name):
        """
        This method removes all non alphabet characters from the string
        """
        name = ''.join([i for i in str(name).strip() if i.isalpha()])
        return name
    
    def remove_adjacent_duplicates(self, name):
        """
        This method replaces repeating characters with only one of such character
        """
        name = ''.join(i for i, _ in itertools.groupby(name))
        return name
    
    def name_cleanup(self, name):
        """
        This method calls other 3 methods to perform cleanup
        args: name
        return: cleaned up name
        """
        name = self.normalize_string(name)
        name = self.keep_alphabets(name)
        name = self.remove_adjacent_duplicates(name)
        return name


# TODO: Fill out this file with information about your package

# HINT: Go back to the object-oriented programming lesson "Putting Code on PyPi" and "Exercise: Upload to PyPi"

# HINT: Here is an example of a setup.py file
# https://packaging.python.org/tutorials/packaging-projects/
from setuptools import setup

setup(name='name_matcher',
      author='Ammar Raza',
      version='0.1',
      descrption='Match fields and provide match percentage',
      packages=['name_matcher'],
      author_email='ammar_reza@hotmail.com',
      zip_safe=False)


from .cleanup import CleanUp

class NameMatcher(CleanUp):
    """
    This class doesn't take any arguments
    """
    def __init__(self):
        """
        initialize parent class's init method via super method
        """
        super().__init__()
        
    def initialzie_to_zero(self):
        """
        This method reinitializes all of the instance arguments back to 0 value
        """
        self.match_percentage = 0
        self.match_count = 0
        self.saved_outcome = 0
        self.saved_match_count = 0
        self.saved_combined_length = 0
        self.combined_lenth = 0
        
    def calculate_match_percentage(self, match_count, combined_length):
        """
        This method calculates the percentage of characters matched between the two strings
        args: match_count, combined_length
        return: percentage of characters matched
        """
        percentage = (match_count/combined_length) * 100
        return percentage
    
    def zip_match_names(self, name1, name2):
        """
        This method takes in two names, returns the number of characters matched and total number of characters in both strings
        args: name1, name2
        return: matched_count, combined_length
        """
        self.combined_length = len(name1) + len(name2)
        zipped = zip(name1, name2)
        for i in zipped:
            if i[0]==i[1]:
                self.match_count += 2
        outcome = self.calculate_match_percentage(self.match_count, self.combined_length)
        if outcome < 90 and len(name1)!=len(name2) and outcome>self.saved_outcome:
            self.saved_outcome = outcome
            self.saved_match_count = self.match_count
            self.saved_combined_length = self.combined_length
            self.match_count = 0
            self.combined_length = len(name1) + len(name2)
            zipped = zip(name1, name2)
            remove_one_char = True
            for i, j in enumerate(zipped):
                if j[0]!=j[1] and len(name1)>len(name2) and remove_one_char:
                    name1 = name1[:i]+name1[i+1:]
                    remove_one_char = False
                elif j[0]!=j[1] and len(name1)<len(name2) and remove_one_char:
                    name2 = name2[:i]+name2[i+1:]
                    remove_one_char = False
            self.match_count, self.combined_length = self.zip_match_names(name1, name2)
        elif outcome<self.saved_outcome:
            self.match_count = self.saved_match_count
            self.combined_length = self.saved_combined_length
        return self.match_count, self.combined_length
    
    def name_matcher(self, name1, name2):
        """
        This method takes two names and calls all the methods to perform cleanup, string match and match percentage calculation
        args: name1, name2
        return: match_percentage
        """
        self.initialzie_to_zero()
        name1 = self.name_cleanup(name1)
        name2 = self.name_cleanup(name2)
        self.match_count, self.combined_length = self.zip_match_names(name1, name2)
        self.match_percentage = self.calculate_match_percentage(self.match_count, self.combined_length)
        return self.match_percentage